/*============================================================================
  File:     01_Stats and the Optimizer.sql

  SQL Server Versions: 2012 onwards
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2015, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/


USE [AdventureWorks2014];
GO


/*
	What's the easiest way to view the statistics for a table?
*/

/*
	old way...it's deprecated
*/
EXEC sp_helpstats 'Sales.SalesOrderDetail', 'ALL';
GO

/*
	use sys.stats...and some other catalog views
	available in SQL 2005+
	Just a reminder...index stats vs. column stats
*/
SELECT  [sch].[name] + '.' + [so].[name] AS [TableName] ,
        [si].[index_id] AS [Index ID] ,
        [ss].[name] AS [Statistic] ,
        STUFF(( SELECT  ', ' + [c].[name]
                FROM    [sys].[stats_columns] [sc]
                        JOIN [sys].[columns] [c] ON [c].[column_id] = [sc].[column_id]
                                                    AND [c].[object_id] = [sc].[OBJECT_ID]
                WHERE   [sc].[object_id] = [ss].[object_id]
                        AND [sc].[stats_id] = [ss].[stats_id]
                ORDER BY [sc].[stats_column_id]
              FOR
                XML PATH('')
              ), 1, 2, '') AS [ColumnsInStatistic] ,
        [ss].[auto_Created] AS [WasAutoCreated] ,
        [ss].[user_created] AS [WasUserCreated] ,
        [ss].[has_filter] AS [IsFiltered] ,
        [ss].[filter_definition] AS [FilterDefinition] ,
        [ss].[is_temporary] AS [IsTemporary]
FROM    [sys].[stats] [ss]
        JOIN [sys].[objects] AS [so] ON [ss].[object_id] = [so].[object_id]
        JOIN [sys].[schemas] AS [sch] ON [so].[schema_id] = [sch].[schema_id]
        LEFT OUTER JOIN [sys].[indexes] AS [si] ON [so].[object_id] = [si].[object_id]
                                                   AND [ss].[name] = [si].[name]
WHERE   [so].[object_id] = OBJECT_ID(N'Sales.SalesOrderDetail')
ORDER BY [ss].[user_created] ,
        [ss].[auto_created] ,
        [ss].[has_filter];
GO


/*
	What's the easiest way to view the indexes for a table?
*/
sp_helpindex 'Sales.SalesOrderDetail'

/*
	Kimberly's sp_helpindex: http://www.sqlskills.com/blogs/kimberly/category/sp_helpindex-rewrites/
*/
sp_SQLskills_SQL2012_helpindex 'Sales.SalesOrderDetail'



/*
	to view information about a specific statistic
*/

DBCC SHOW_STATISTICS ('Person.Person', IX_Person_LastName_FirstName_MiddleName) WITH HISTOGRAM;
GO

DBCC SHOW_STATISTICS ('Sales.SalesOrderDetail', IX_SalesOrderDetail_ProductID);
GO

DBCC SHOW_STATISTICS ('Sales.SalesOrderDetail', IX_SalesOrderDetail_ProductID) WITH STAT_HEADER;
GO

DBCC SHOW_STATISTICS ('Sales.SalesOrderDetail',IX_SalesOrderDetail_ProductID) WITH DENSITY_VECTOR;
GO

DBCC SHOW_STATISTICS ('Sales.SalesOrderDetail',IX_SalesOrderDetail_ProductID) WITH HISTOGRAM;
GO



/*
	DMF only exists in 2008R2 SP2 and 2012 SP1
*/
SELECT
    sch.name + '.' + so.name AS "Table",
    ss.name AS "Statistic",
	ss.auto_Created AS "Auto Created?",
	ss.user_created AS "User Created?",
	ss.has_filter AS "Filtered?", 
    ss.filter_definition AS "Filter Definition", 
    sp.last_updated AS "Stats Last Updated", 
    sp.rows AS "Rows in Table", 
    sp.rows_sampled AS "Rows Sampled", 
    sp.unfiltered_rows AS "Unfiltered Rows",
	sp.modification_counter AS "Row Modifications",
	sp.steps AS "Histogram Steps"
FROM sys.stats ss
JOIN sys.objects so ON ss.object_id = so.object_id
JOIN sys.schemas sch ON so.schema_id = sch.schema_id
CROSS APPLY sys.dm_db_stats_properties(so.object_id, ss.stats_id) AS sp
WHERE so.object_id = object_id('Sales.SalesOrderDetail')
ORDER BY ss.user_created, ss.auto_created, ss.has_filter;
GO


/*
	So now we know what indexes and stats exist...how does the optimizer use this information?
*/


/*
	Here's a simple query...
	*enable execution plan
	What do we see for estimated number of rows?  
	How does that compare to the actual?
*/
SELECT [ProductID], [OrderQty]
FROM [Sales].[SalesOrderDetail]
WHERE [ProductID] = 732
ORDER BY [ProductID];

/*  
	What does the histogram say?
*/
DBCC SHOW_STATISTICS ('Sales.SalesOrderDetail', IX_SalesOrderDetail_ProductID);
GO


/*
	A different value?
*/
SELECT [ProductID], [OrderQty]
FROM [Sales].[SalesOrderDetail]
WHERE [ProductID] = 840
ORDER BY [ProductID];

/*
	Why is the estimate different than the actual?
*/
DBCC SHOW_STATISTICS ('Sales.SalesOrderDetail', IX_SalesOrderDetail_ProductID);
GO

SELECT (COUNT(ProductID))
FROM [Sales].[SalesOrderDetail]
WHERE [ProductID] BETWEEN 839 AND 842;

SELECT [ProductID], COUNT(*)
FROM [Sales].[SalesOrderDetail]
WHERE [ProductID] BETWEEN 839 AND 842
GROUP BY [ProductID]
ORDER BY [ProductID];



/* go to 02 script */