/*============================================================================
  File:     02_Updating Statistics.sql

  SQL Server Versions: 2012 onwards
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2015, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE [AdventureWorks2014];
GO

/*
	working queries
	*enabled plan
*/
SELECT [ProductID], [OrderQty]
FROM [Sales].[SalesOrderDetail]
WHERE [ProductID] = 739
ORDER BY [ProductID];
GO

SELECT [ProductID], [OrderQty]
FROM [Sales].[SalesOrderDetail]
WHERE [ProductID] = 840
ORDER BY [ProductID];
GO

SELECT [ProductID], [OrderQty]
FROM [Sales].[SalesOrderDetail]
WHERE [ProductID] = 890
ORDER BY [ProductID];
GO

SELECT [ProductID], [OrderQty]
FROM [Sales].[SalesOrderDetail]
WHERE [ProductID] = 871
ORDER BY [ProductID];
GO

/*
	what plan exists in cache?
*/
SELECT qs.execution_count, s.text, cp.size_in_bytes, qs.query_hash, qs.query_plan_hash, qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
JOIN sys.dm_exec_cached_plans cp on qs.plan_handle = cp.plan_handle
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
WHERE s.text LIKE '%SalesOrderDetail%';



/*
	For each variation, one plan in cache!
	This can bloat the plan cache...
	We can reduce this via a stored procedure
*/
IF OBJECT_ID ( 'Sales.usp_GetProductInfo', 'P' ) IS NOT NULL 
    DROP PROCEDURE [Sales].[usp_GetProductInfo];
GO

CREATE PROCEDURE [Sales].[usp_GetProductInfo]
	@ProductID INT
AS	
	SET NOCOUNT ON;
	SELECT [ProductID], [OrderQty]
	FROM [Sales].[SalesOrderDetail]
	WHERE [ProductID] = @ProductID
	ORDER BY [ProductID];

GO

SET STATISTICS IO ON;

EXECUTE Sales.usp_GetProductInfo 739;
GO

EXECUTE Sales.usp_GetProductInfo 840; 
GO

EXECUTE Sales.usp_GetProductInfo 890;
GO

EXECUTE Sales.usp_GetProductInfo 871;
GO

EXECUTE Sales.usp_GetProductInfo 900;
GO

EXECUTE Sales.usp_GetProductInfo 870;
GO

/*
	Note I/O here:

*/


/*
	what exists in cache now?
*/
SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, cp.size_in_bytes/1024 [PlanSizeKB], qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
JOIN sys.dm_exec_cached_plans cp ON qs.plan_handle = cp.plan_handle
WHERE s.text LIKE '%SalesOrderDetail%';




/*
	What happens as data changes in the table, or data is added?
	add some data
*/
BULK INSERT [AdventureWorks2014].[Sales].[SalesOrderDetail]
FROM 'C:\SQLskills\General\sod.txt'
	WITH
	(
	DATAFILETYPE = 'native',
	TABLOCK
	);


/*
	run the procedure...
*/
EXECUTE Sales.usp_GetProductInfo 870;


/*
	what exists in cache now?
*/
SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, cp.size_in_bytes/1024 [PlanSizeKB], qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
JOIN sys.dm_exec_cached_plans cp ON qs.plan_handle = cp.plan_handle
WHERE s.text LIKE '%SalesOrderDetail%';





/*
	Plan recompiled!
	And...the plan changed
	Verify, auto-update stats enabled
*/
SELECT [name], [is_auto_update_stats_on]
FROM [sys].[databases] 
WHERE [name] = 'AdventureWorks2014'



/*
	How does this affect query performance?
*/
SET STATISTICS IO ON;

EXECUTE Sales.usp_GetProductInfo 870;
GO

EXECUTE Sales.usp_GetProductInfo 739;
GO

EXECUTE Sales.usp_GetProductInfo 840; 
GO

EXECUTE Sales.usp_GetProductInfo 890;
GO

EXECUTE Sales.usp_GetProductInfo 871;
GO

EXECUTE Sales.usp_GetProductInfo 900;
GO

/*
	I/O output


*/
