/*============================================================================
  File:     2008R2_StatsUpdates.sql

  SQL Server Versions: 2008R2 and below
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2015, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/*
	ensure Auto Update Statistics is DISABLED
*/
ALTER DATABASE [AdventureWorks2008R2] SET AUTO_UPDATE_STATISTICS OFF;


/*
	create a new table with indexes for testing
*/
USE [AdventureWorks2008R2];
GO

IF EXISTS (SELECT * FROM [sys].[objects] WHERE [name] = 'TestSalesOrderDetail' AND [type] = 'U' )
BEGIN	
	DROP TABLE [Sales].[TestSalesOrderDetail];
END

SELECT *
INTO [Sales].[TestSalesOrderDetail]
FROM [Sales].[SalesOrderDetail]
WHERE [SalesOrderDetailID] <= 121317;
GO

CREATE CLUSTERED INDEX [PK_TestSalesOrderDetail_SalesOrderID_SalesOrderDetailID] ON [Sales].[TestSalesOrderDetail] ([SalesOrderID], [SalesOrderDetailID]);
GO

CREATE UNIQUE NONCLUSTERED INDEX [AK_TestSalesOrderDetail_rowguid] ON [Sales].[TestSalesOrderDetail] ([rowguid]);
GO

CREATE NONCLUSTERED INDEX [IX_TestSalesOrderDetail_ProductID] ON [Sales].[TestSalesOrderDetail] ([ProductID]);
GO

DBCC SHOW_STATISTICS ("Sales.TestSalesOrderDetail",IX_TestSalesOrderDetail_ProductID)


/*
	create the same procedure on our TEST table
*/
IF OBJECT_ID ( 'Sales.usp_TESTGetProductInfo', 'P' ) IS NOT NULL 
    DROP PROCEDURE [Sales].[usp_TESTGetProductInfo];
GO

CREATE PROCEDURE [Sales].[usp_TestGetProductInfo]
	@ProductID INT
AS	
	SET NOCOUNT ON;
	SELECT [ProductID], [OrderQty]
	FROM [Sales].[TestSalesOrderDetail]
	WHERE [ProductID] = @ProductID
	ORDER BY [ProductID];

GO


/*
	enable execution plan
*/
SET STATISTICS IO ON;
EXECUTE Sales.usp_TestGetProductInfo 732;
GO
EXECUTE Sales.usp_TestGetProductInfo 733;
GO
EXECUTE Sales.usp_TestGetProductInfo 870;
GO

SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
WHERE s.text LIKE '%usp_TestGetProductInfo%';



/*
	add some data
*/
BULK INSERT [AdventureWorks2008R2].[Sales].[TestSalesOrderDetail]
FROM 'C:\SQLskills\General\2008_sod_data.txt'
	WITH
	(
	DATAFILETYPE = 'native',
	TABLOCK
	);


/*
	stats haven't updated at this point
*/
DBCC SHOW_STATISTICS ("Sales.TestSalesOrderDetail",IX_TestSalesOrderDetail_ProductID);


/*
	This WOULD invoke the update if Auto Update Statistics was ENABLED
*/
EXECUTE Sales.usp_TestGetProductInfo 732;


-- check execution count
SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
WHERE s.text LIKE '%usp_TestGetProductInfo%';

/*
	check stats again
*/
DBCC SHOW_STATISTICS ("Sales.TestSalesOrderDetail",IX_TestSalesOrderDetail_ProductID);



SELECT [name], [is_auto_update_stats_on]
FROM [sys].[databases]
WHERE [database_id] = DB_ID('AdventureWorks2008R2');


UPDATE STATISTICS Sales.TestSalesOrderDetail WITH FULLSCAN;

DBCC SHOW_STATISTICS ("Sales.TestSalesOrderDetail",IX_TestSalesOrderDetail_ProductID);



SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
WHERE s.text LIKE '%usp_TestGetProductInfo%';


EXECUTE Sales.usp_TestGetProductInfo 733;
GO
EXECUTE Sales.usp_TestGetProductInfo 777;
GO
EXECUTE Sales.usp_TestGetProductInfo 870;
GO

SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
WHERE s.text LIKE '%usp_TestGetProductInfo%';


/*
	This behavior was a bug...

	FROM: https://connect.microsoft.com/SQLServer/feedback/details/769338/update-statistics-does-not-cause-plan-invalidation#

	"...the old behavior of requiring auto stats to be enabled in order for a manual update 
	to cause recompile was really just a bug."

*/



/*
	ENABLE Auto Update Statistics 
*/
ALTER DATABASE [AdventureWorks2008R2] SET AUTO_UPDATE_STATISTICS ON;

/*
	check the cache...it cleared it
*/
SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
WHERE s.text LIKE '%usp_TestGetProductInfo%';

/* 
	Run the SP, check cache again
*/
EXECUTE Sales.usp_TestGetProductInfo 733;
GO
EXECUTE Sales.usp_TestGetProductInfo 777;
GO
EXECUTE Sales.usp_TestGetProductInfo 870;
GO

SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
WHERE s.text LIKE '%usp_TestGetProductInfo%';
GO

/*
	Load more data in
*/
BULK INSERT [AdventureWorks2008R2].[Sales].[TestSalesOrderDetail]
FROM 'C:\SQLskills\General\sod3.txt'
	WITH
	(
	DATAFILETYPE = 'native',
	TABLOCK
	);


/*
	stats haven't updated at this point
*/
DBCC SHOW_STATISTICS ("Sales.TestSalesOrderDetail",IX_TestSalesOrderDetail_ProductID);


/*
	This should invoke the update 
*/
EXECUTE Sales.usp_TestGetProductInfo 732;


/*
	check execution count and stats
*/
SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
WHERE s.text LIKE '%usp_TestGetProductInfo%';


DBCC SHOW_STATISTICS ("Sales.TestSalesOrderDetail",IX_TestSalesOrderDetail_ProductID);
GO

